#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals, print_function

import os
import sys
import requests
import datetime
import subprocess
import inspect
import multiprocessing

import json

import traceback
import tempfile

DATE_FORMAT = "%Y-%m-%d %H:%M:%S"

__version__ = "1.0.5"


def _json_date_hook(json_dict):
    for (key, value) in json_dict.items():
        try:
            json_dict[key] = datetime.datetime.strptime(value, DATE_FORMAT)
        except Exception:
            pass
    return json_dict


def _get_last_status(state_file):
    try:
        with open(state_file, 'rb') as f:
            last_status = json.load(f, object_hook=_json_date_hook)
    except Exception:
        last_status = {'lastRun': datetime.datetime(2000, 1, 1),
                       'lastFullRun': datetime.datetime(2000, 1, 1)}

    return last_status


class InfoPlugin(object):

    def getValue(self):
        raise NotImplementedError


class RunCommand(InfoPlugin):
    # params: ejecutable y parámetros para pasar a check_output
    #         Ej: ["/bin/uname", "-r"]
    def getValue(self):
        stdOut = tempfile.TemporaryFile(mode='w+t')

        try:
            retCode = subprocess.call(self.params, stdout=stdOut)
        except Exception:
            retCode = 1

        if retCode != 0:
            return "ERROR: falló el comando '%s': %d" % (" ".join(self.params), retCode)
        else:
            stdOut.seek(0)
            return stdOut.read().strip()


class Q1234Version(RunCommand):
    key = "q1234-client"

    def getValue(self):
        return __version__


class KernelVersion(RunCommand):
    key = "kernel"
    params = ["/bin/uname", "-r"]


class KernelFullVersion(RunCommand):
    key = "kernel-full"
    params = ["/bin/uname", "-a"]


class PythonVersion(InfoPlugin):
    key = "python"

    def getValue(self):
        return ".".join(map(str, sys.version_info[:3]))


class Fierro(InfoPlugin):
    key = "fierro"

    def getValue(self):
        ret = {}
        for x in os.listdir("/opt"):
            if not x.startswith("fierro"):
                continue
            fullName = os.path.join("/opt", x)
            if not os.path.islink(fullName):
                continue
            ret[x[len("fierro"):]] = os.path.realpath(fullName).split("-")[-1]
        return ret


class CheckFierroBackup(InfoPlugin):
    key = "fierroBackup"

    CONFFILE = "/etc/opt/fierroutils/dumpDbFierro.conf"

    def _readConfigFile(self, fileName):
        """Lee un archivo de configuración de Bash y devuelve los valores como diccionario
           VARIABLE1=valor1
           VARIABLE2=valor2

           --> {"VARIABLE1": "valor1", "VARIABLE2": "valor2"}
        """
        ret = {}
        for line in open(fileName, 'rt'):
            if "=" in line:
                idx = line.index("=")
                value = line[idx + 1:].strip()
                if value and value[0] == "'":
                    value = value.strip("'")
                elif value and value[0] == '"':
                    value = value.strip('"')
                ret[line[0:idx].strip()] = value
        return ret

    def getValue(self):
        ret = {}
        if os.path.exists(self.CONFFILE):
            confVars = self._readConfigFile(self.CONFFILE)
            dumpDir = confVars.get("DUMPDIR", "/home/fierro/dbDumps")
            ret["dumpDir"] = dumpDir
            lastDump = os.path.join(dumpDir, "ultimodump")
            if not os.path.exists(lastDump):
                return "No existe el directorio de backups"
            lastDumpFileName = os.listdir(lastDump)
            if lastDumpFileName:
                if len(lastDumpFileName) > 1:
                    ret["lastDump"] = "Hay más de uno"
                else:
                    ret["lastDump"] = lastDumpFileName[0]
                    size = os.path.getsize(os.path.join(lastDump, lastDumpFileName[0]))
                    if size < 100:
                        ret["lastDumpSize"] = "VACIO!!!"
                    else:
                        ret["lastDumpSize"] = "%.1f Mb" % (size / (1024.0 * 1024.0))
        else:
            return "No está configurado"
        return ret


class MemoryInfo(InfoPlugin):
    key = "memory"

    def getValue(self):
        mem_bytes = os.sysconf("SC_PAGE_SIZE") * os.sysconf("SC_PHYS_PAGES")
        return mem_bytes


class CPUInfo(InfoPlugin):
    key = "cpu"

    def getValue(self):
        ret = {}
        for line in open("/proc/cpuinfo"):
            if line.startswith("model name\t"):
                ret["model"] = line.split(":", 1)[1].strip()
                break
        ret["cores"] = multiprocessing.cpu_count()
        return ret


def _load_extension_data(basedir, filter_extension='.ext'):
    if not os.path.isdir(basedir):
        return {}

    extension_data = {}
    for filename in os.listdir(basedir):
        basename, extension = os.path.splitext(filename)

        if extension != filter_extension:
            continue

        fullpath = os.path.join(basedir, filename)

        try:
            with open(fullpath, 'rb') as f:
                data = json.load(f)
        except Exception as e:
            extension_data[basename] = {"error": str(e)}
            continue

        extension_data[basename] = data
    return extension_data


if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Usage: %s <config-file>" % sys.argv[0], file=sys.stderr)
        sys.exit(2)

    with open(sys.argv[1], 'rb') as f:
        exec(f.read(), globals(), locals())

    if not globals().get("HOST") or not globals().get("PASSWORD"):
        print("Debe configurar HOST y PASSWORD", file=sys.stderr)
        sys.exit(3)

    HOST = globals().get("HOST")
    PASSWORD = globals().get("PASSWORD")
    SERVER = globals().get("SERVER", "server.q123.com.ar")
    PORT = globals().get("PORT", 80)
    STATE_FILE = globals().get("STATE_FILE", "/var/run/q1234-client.state")
    FULL_FREQUENCE = globals().get("FULL_FREQUENCE", 60 * 60 * 2)  # Dos horas
    DISABLED_PLUGINS = globals().get("DISABLED_PLUGINS", [])
    EXTENSION_DIR = globals().get("EXTENSION_DIR", "/etc/q1234client.d/")  # Dos horas

    params = {"password": PASSWORD}

    url = "http://%s:%s/host/%s" % (SERVER, PORT, HOST)

    lastStatus = _get_last_status(STATE_FILE)

    if (datetime.datetime.now() - lastStatus["lastFullRun"]).seconds > FULL_FREQUENCE:
        # Los datos completos no se envían siempre sino cada FULL_FREQUENCE segundos
        full = True
        data = {}
        for c in filter(inspect.isclass, list(globals().values())):
            if issubclass(c, InfoPlugin) and hasattr(c, "key"):
                if c.key in DISABLED_PLUGINS:
                    continue
                obj = c()
                try:
                    data[obj.key] = obj.getValue()
                except Exception:
                    data[obj.key] = "ERROR: %s" % traceback.format_exc()
        data['extension'] = _load_extension_data(EXTENSION_DIR)

        params["data"] = json.dumps(data)
    else:
        full = False

    try:
        response = requests.post(url, data=params)
    except requests.exceptions.HTTPError as e:
        print("Error HTTP enviando la información al servidor", file=sys.stderr)
        print("Código: %s, Mensaje: %s" % (e.code, e.msg), file=sys.stderr)
        sys.stderr.writelines(e.fp)
        sys.exit(1)

    lastStatus["lastRun"] = datetime.datetime.now()
    if full:
        lastStatus["lastFullRun"] = lastStatus["lastRun"]
    for k in ['lastRun', 'lastFullRun']:
        lastStatus[k] = lastStatus[k].strftime(DATE_FORMAT)

    with open(STATE_FILE, "wt") as f:
        json.dump(lastStatus, f)
